@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -33,7 +33,7 @@ from nuitka.finalizations.FinalizeMarkups import getImportedNames
 from nuitka.freezer.IncludedEntryPoints import (
     addIncludedEntryPoints,
     addShlibEntryPoint,
-    getStandardEntryPoints,
+    getStandaloneEntryPoints,
     setMainEntryPoint,
 )
 from nuitka.freezer.Standalone import copyDataFiles
@@ -41,6 +41,11 @@ from nuitka.importing import Importing, Recursion
 from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PostProcessing import executePostProcessing
+from nuitka.Progress import (
+    closeProgressBar,
+    reportProgressBar,
+    setupProgressBar,
+)
 from nuitka.PythonVersions import (
     getPythonABI,
     getSupportedPythonVersions,
@@ -70,7 +75,7 @@ from .optimizations import Optimization
 from .tree import Building
 
 
-def createNodeTree(filename):
+def _createNodeTree(filename):
     """Create a node tree.
 
     Turn that source code into a node tree structure. If recursion into
@@ -120,7 +125,10 @@ def createNodeTree(filename):
         )
 
         if kind != "absolute":
-            sys.exit("Error, failed to locate package %r." % package_name)
+            inclusion_logger.sysexit(
+                "Error, failed to locate package %r you asked to include."
+                % package_name
+            )
 
         Recursion.checkPluginPath(
             plugin_filename=package_directory, module_package=package_package
@@ -136,7 +144,9 @@ def createNodeTree(filename):
         )
 
         if kind != "absolute":
-            sys.exit("Error, failed to locate module %r." % module_name)
+            inclusion_logger.sysexit(
+                "Error, failed to locate module %r you asked to include." % module_name
+            )
 
         Recursion.checkPluginSinglePath(
             plugin_filename=module_filename, module_package=module_package
@@ -239,13 +249,11 @@ def pickSourceFilenames(source_dir, modules):
 standalone_entry_points = []
 
 
-def makeSourceDirectory(main_module):
+def makeSourceDirectory():
     """Get the full list of modules imported, create code for all of them."""
     # We deal with a lot of details here, but rather one by one, and split makes
     # no sense, pylint: disable=too-many-branches
 
-    assert main_module.isCompiledPythonModule()
-
     # assert main_module in ModuleRegistry.getDoneModules()
 
     # We might have chosen to include it as bytecode, and only compiled it for
@@ -291,24 +299,12 @@ def makeSourceDirectory(main_module):
         if module.isCompiledPythonModule():
             Finalization.prepareCodeGeneration(module)
 
-    # Pick filenames.
-    source_dir = OutputDirectories.getSourceDirectoryPath()
+    # Do some reporting and determine compiled module to work on
+    compiled_modules = []
 
-    module_filenames = pickSourceFilenames(
-        source_dir=source_dir, modules=ModuleRegistry.getDoneModules()
-    )
-
-    # Generate code for modules.
     for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
-            c_filename = module_filenames[module]
-
-            source_code = CodeGeneration.generateModuleCode(
-                module=module,
-                data_filename=os.path.basename(c_filename + "onst"),  # Really .const
-            )
-
-            writeSourceCode(filename=c_filename, source_code=source_code)
+            compiled_modules.append(module)
 
             if Options.isShowInclusion():
                 inclusion_logger.info(
@@ -329,6 +325,37 @@ def makeSourceDirectory(main_module):
         else:
             assert False, module
 
+    # Pick filenames.
+    source_dir = OutputDirectories.getSourceDirectoryPath()
+
+    module_filenames = pickSourceFilenames(
+        source_dir=source_dir, modules=compiled_modules
+    )
+
+    setupProgressBar(
+        stage="C Source Generation",
+        unit="module",
+        total=len(compiled_modules),
+    )
+
+    # Generate code for compiled modules, this can be slow, so do it separately
+    # with a progress bar.
+    for module in compiled_modules:
+        c_filename = module_filenames[module]
+
+        reportProgressBar(
+            item=module.getFullName(),
+        )
+
+        source_code = CodeGeneration.generateModuleCode(
+            module=module,
+            data_filename=os.path.basename(c_filename + "onst"),  # Really .const
+        )
+
+        writeSourceCode(filename=c_filename, source_code=source_code)
+
+    closeProgressBar()
+
     (
         helper_decl_code,
         helper_impl_code,
@@ -368,7 +395,7 @@ def makeSourceDirectory(main_module):
 def runSconsBackend(quiet):
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches and statements,
-    # pylint: disable=too-many-branches,too-many-statements
+    # pylint: disable=too-many-branches
 
     asBoolStr = SconsInterface.asBoolStr
 
@@ -397,23 +424,15 @@ def runSconsBackend(quiet):
     if not Options.shallMakeModule():
         options["result_exe"] = OutputDirectories.getResultFullpath()
 
-    # Ask Scons to cache on Windows, except where the directory is thrown
-    # away. On non-Windows you can should use ccache instead.
-    if not Options.isRemoveBuildDir() and Utils.getOS() == "Windows":
-        options["cache_mode"] = "true"
-
-    if Options.isLto():
-        options["lto_mode"] = asBoolStr(True)
-
     if Options.shallUseStaticLibPython():
         options["static_libpython"] = asBoolStr(True)
 
-    if Options.shallDisableConsoleWindow():
-        options["win_disable_console"] = asBoolStr(True)
-
     if Options.isStandaloneMode():
         options["standalone_mode"] = asBoolStr(True)
 
+    if Options.isOnefileMode():
+        options["onefile_mode"] = asBoolStr(True)
+
     if Options.shallTreatUninstalledPython():
         options["uninstalled_python"] = asBoolStr(True)
 
@@ -422,27 +441,9 @@ def runSconsBackend(quiet):
             len(ModuleRegistry.getUncompiledTechnicalModules())
         )
 
-    if Options.isShowScons():
-        options["show_scons"] = asBoolStr(True)
-
-    if Options.isMingw64():
-        options["mingw_mode"] = asBoolStr(True)
-
-    if Options.getMsvcVersion():
-        msvc_version = Options.getMsvcVersion()
-
-        msvc_version = msvc_version.replace("exp", "Exp")
-        if "." not in msvc_version:
-            msvc_version += ".0"
-
-        options["msvc_version"] = msvc_version
-
     if Utils.getOS() == "Windows":
         options["noelf_mode"] = asBoolStr(True)
 
-    if Options.isClang():
-        options["clang_mode"] = asBoolStr(True)
-
     if Options.isProfile():
         options["profile_mode"] = asBoolStr(True)
 
@@ -452,10 +453,12 @@ def runSconsBackend(quiet):
     if "no_asserts" in getPythonFlags():
         options["python_sysflag_optimize"] = asBoolStr(True)
 
-    if python_version < 300 and sys.flags.py3k_warning:
+    if python_version < 0x300 and sys.flags.py3k_warning:
         options["python_sysflag_py3k_warning"] = asBoolStr(True)
 
-    if python_version < 300 and (sys.flags.division_warning or sys.flags.py3k_warning):
+    if python_version < 0x300 and (
+        sys.flags.division_warning or sys.flags.py3k_warning
+    ):
         options["python_sysflag_division_warning"] = asBoolStr(True)
 
     if sys.flags.bytes_warning:
@@ -470,35 +473,20 @@ def runSconsBackend(quiet):
     if "no_randomization" in Options.getPythonFlags():
         options["python_sysflag_no_randomization"] = asBoolStr(True)
 
-    if python_version < 300 and sys.flags.unicode:
+    if python_version < 0x300 and sys.flags.unicode:
         options["python_sysflag_unicode"] = asBoolStr(True)
 
-    if python_version >= 370 and sys.flags.utf8_mode:
+    if python_version >= 0x370 and sys.flags.utf8_mode:
         options["python_sysflag_utf8"] = asBoolStr(True)
 
     abiflags = getPythonABI()
     if abiflags:
         options["abiflags"] = abiflags
 
-    cpp_defines = Plugins.getPreprocessorSymbols()
-    if cpp_defines:
-        options["cpp_defines"] = ",".join(
-            "%s%s%s" % (key, "=" if value else "", value or "")
-            for key, value in cpp_defines.items()
-        )
-
-    link_libraries = Plugins.getExtraLinkLibraries()
-    if link_libraries:
-        options["link_libraries"] = ",".join(link_libraries)
-
     if Options.shallMakeModule():
         options["module_suffix"] = getSharedLibrarySuffix(preferred=True)
 
-    if Options.shallRunInDebugger():
-        options["full_names"] = asBoolStr(True)
-
-    if Options.assumeYesForDownloads():
-        options["assume_yes_for_downloads"] = asBoolStr(True)
+    SconsInterface.setCommonOptions(options)
 
     return (
         SconsInterface.runScons(
@@ -528,7 +516,7 @@ def writeBinaryData(filename, binary_data):
 
 
 def callExecPython(args, clean_path, add_path):
-    old_python_path = os.environ.get("PYTHONPATH", None)
+    old_python_path = os.environ.get("PYTHONPATH")
 
     if clean_path and old_python_path is not None:
         os.environ["PYTHONPATH"] = ""
@@ -571,13 +559,13 @@ def executeModule(tree, clean_path):
     callExecPython(clean_path=clean_path, add_path=True, args=args)
 
 
-def compileTree(main_module):
+def compileTree():
     source_dir = OutputDirectories.getSourceDirectoryPath()
 
     general.info("Completed Python level compilation and optimization.")
 
     if not Options.shallOnlyExecCCompilerCall():
-        general.info("Generating C source code for backend compiler.")
+        general.info("Generating source code for C backend compiler.")
 
         if Options.isShowProgress() or Options.isShowMemory():
             general.info(
@@ -586,7 +574,7 @@ def compileTree(main_module):
                 )
             )
         # Now build the target language code for the whole tree.
-        makeSourceDirectory(main_module=main_module)
+        makeSourceDirectory()
 
         bytecode_accessor = ConstantAccessor(
             data_filename="__bytecode.const", top_level_name="bytecode_data"
@@ -604,7 +592,7 @@ def compileTree(main_module):
         source_dir = OutputDirectories.getSourceDirectoryPath()
 
         if not os.path.isfile(os.path.join(source_dir, "__helpers.h")):
-            sys.exit("Error, no previous build directory exists.")
+            general.sysexit("Error, no previous build directory exists.")
 
     if Options.isShowProgress() or Options.isShowMemory():
         general.info(
@@ -643,7 +631,7 @@ def handleSyntaxError(e):
     error_message = SyntaxErrors.formatOutput(e)
 
     if not Options.is_fullcompat:
-        if python_version < 300:
+        if python_version < 0x300:
             suggested_python_version_str = getSupportedPythonVersions()[-1]
         else:
             suggested_python_version_str = "2.7"
@@ -674,9 +662,9 @@ def main():
     """
 
     # Main has to fulfill many options, leading to many branches and statements
-    # to deal with them.  pylint: disable=too-many-branches
-
-    general.info("Starting Python compilation.")
+    # to deal with them.  pylint: disable=too-many-branches,too-many-statements
+    if not Options.shallDumpBuiltTreeXML():
+        general.info("Starting Python compilation.")
 
     filename = Options.getPositionalArgs()[0]
 
@@ -688,7 +676,7 @@ def main():
 
     # Turn that source code into a node tree structure.
     try:
-        main_module = createNodeTree(filename=filename)
+        main_module = _createNodeTree(filename=filename)
     except (SyntaxError, IndentationError) as e:
         handleSyntaxError(e)
 
@@ -698,7 +686,7 @@ def main():
             dumpTreeXML(module)
     else:
         # Make the actual compilation.
-        result, options = compileTree(main_module=main_module)
+        result, options = compileTree()
 
         # Exit if compilation failed.
         if not result:
@@ -712,12 +700,6 @@ def main():
 
         executePostProcessing()
 
-        # Remove the source directory (now build directory too) if asked to.
-        if Options.isRemoveBuildDir():
-            removeDirectory(
-                path=OutputDirectories.getSourceDirectoryPath(), ignore_errors=False
-            )
-
         if Options.shallMakeModule() and Options.shallCreatePyiFile():
             pyi_filename = OutputDirectories.getResultBasepath() + ".pyi"
 
@@ -763,7 +745,7 @@ __name__ = ...
             copyUsedDLLs(
                 source_dir=OutputDirectories.getSourceDirectoryPath(),
                 dist_dir=dist_dir,
-                standalone_entry_points=getStandardEntryPoints(),
+                standalone_entry_points=getStandaloneEntryPoints(),
             )
 
             copyDataFiles(dist_dir=dist_dir)
@@ -773,13 +755,39 @@ __name__ = ...
             if Options.isOnefileMode():
                 packDistFolderToOnefile(dist_dir, binary_filename)
 
-        general.info(
-            "Successfully created %r."
-            % OutputDirectories.getResultFullpath(onefile=Options.isOnefileMode())
+                if Options.isRemoveBuildDir():
+                    general.info("Removing dist folder %r." % dist_dir)
+
+                    removeDirectory(path=dist_dir, ignore_errors=False)
+                else:
+                    general.info(
+                        "Keeping dist folder %r for inspection, no need to use it."
+                        % dist_dir
+                    )
+
+        # Remove the source directory (now build directory too) if asked to.
+        source_dir = OutputDirectories.getSourceDirectoryPath()
+
+        if Options.isRemoveBuildDir():
+            general.info("Removing build directory %r." % source_dir)
+
+            removeDirectory(path=source_dir, ignore_errors=False)
+            assert not os.path.exists(source_dir)
+        else:
+            general.info("Keeping build directory %r." % source_dir)
+
+        final_filename = OutputDirectories.getResultFullpath(
+            onefile=Options.isOnefileMode()
         )
 
+        Plugins.onFinalResult(final_filename)
+
+        general.info("Successfully created %r." % final_filename)
+
         # Execute the module immediately if option was given.
         if Options.shallExecuteImmediately():
+            general.info("Launching %r." % final_filename)
+
             if Options.shallMakeModule():
                 executeModule(
                     tree=main_module,
@@ -787,8 +795,6 @@ __name__ = ...
                 )
             else:
                 executeMain(
-                    binary_filename=OutputDirectories.getResultFullpath(
-                        onefile=Options.isOnefileMode()
-                    ),
+                    binary_filename=final_filename,
                     clean_path=Options.shallClearPythonPathEnvironment(),
                 )
